package jumpstart.min.business.domain.security;

import java.util.Date;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.PostLoad;
import javax.persistence.PrePersist;
import javax.persistence.PreRemove;
import javax.persistence.PreUpdate;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;
import javax.persistence.UniqueConstraint;

import jumpstart.min.business.commons.exception.BusinessException;
import jumpstart.min.business.commons.exception.GenericBusinessException;
import jumpstart.min.business.commons.exception.ValueRequiredException;
import jumpstart.min.business.domain.base.BaseEntityAutoGenId;
import jumpstart.min.util.StringUtil;

/**
 * The User entity.
 * 
 * @version Revision: 0.1, Date: 10/06/2006
 */
@Entity
@Table(name = "user_min", uniqueConstraints = { @UniqueConstraint(columnNames = { "login_id" }) })
@SuppressWarnings("serial")
public class User extends BaseEntityAutoGenId {
//	static private Log LOG = LogFactory.getLog(User.class);

	static public final String ADMIN_LOGINID = "admin";

	static public final String[] salutations = { "", "Ms", "Mrs", "Mr", "Dr", "Prof" };

	@Column(name = "login_id", length = 15, nullable = false)
	protected String loginId;

	@Column(name = "password", length = 32)
	protected String password;

	@Column(name = "salutation", length = 6)
	protected String salutation;

	@Column(name = "first_name", length = 20, nullable = false)
	protected String firstName;

	@Column(name = "last_name", length = 20, nullable = false)
	protected String lastName;

	@Column(name = "email_address", length = 80)
	protected String emailAddress;

	@Column(name = "active")
	protected boolean active;

	@Column(name = "expiration_date")
	@Temporal(TemporalType.DATE)
	protected Date expirationDate;

	@Transient
	protected transient String _loaded_password = null;

	public String toString() {
		StringBuffer buf = new StringBuffer();
		buf.append("User: [");
		buf.append("id=" + id + ", ");
		buf.append("loginId=" + loginId + ", ");
		buf.append("password=" + password + ", ");
		buf.append("salutation=" + salutation + ", ");
		buf.append("firstName=" + firstName + ", ");
		buf.append("lastName=" + lastName + ", ");
		buf.append("emailAddress=" + emailAddress + ", ");
		buf.append("expirationDate=" + expirationDate + ", ");
		buf.append("active=" + active + ", ");
		buf.append("version=" + version);
		buf.append("]");
		return buf.toString();
	}

	@PrePersist
	void prePersistOfUser() throws BusinessException {
		validateUser();
	}

	@PostLoad
	void postLoadOfUser() {
		_loaded_password = password;
	}

	@PreUpdate
	void preUpdateOfUser() throws BusinessException {
		validateUser();
	}

	@PreRemove
	void preRemoveOfUser() throws BusinessException {
		// Check business rules here, eg.
		// if (entity.getParts().size() > 0) {
		// throw new CannotDeleteIsNotEmptyException(this, id, "Part");
		// }

		// There's no need to remove me from the parent(s) - that's the caller's
		// responsibility (and for performance, it might not bother)
	}

	public void validateUser() throws BusinessException {

		// Validate syntax...

		if (StringUtil.isEmpty(loginId)) {
			throw new ValueRequiredException(this, "User_loginId");
		}

		if (StringUtil.isEmpty(firstName)) {
			throw new ValueRequiredException(this, "User_firstName");
		}

		if (StringUtil.isEmpty(lastName)) {
			throw new ValueRequiredException(this, "User_lastName");
		}

		if (StringUtil.isEmpty(password)) {
			throw new ValueRequiredException(this, "User_password");
		}

		// Validate semantics...

		if (expirationDate != null && loginId.equals(ADMIN_LOGINID)) {
			throw new GenericBusinessException("User_expirationdate_not_permitted_for_user",
					new Object[] { ADMIN_LOGINID });
		}

	}

	public void authenticate(String password) throws BusinessException {

		if (password == null || !password.equals(this.password)) {
			throw new GenericBusinessException("User_password_incorrect");
		}

	}

	/**
	 * This method provides a way for users to change their own passwords.
	 */
	void changePassword(String currentPassword, String newPassword) throws BusinessException {

		if (this.password != null && !currentPassword.equals(this.password)) {
			throw new GenericBusinessException("User_password_incorrect");
		}

		setPassword(newPassword);
	}

	public String getLoginId() {
		return loginId;
	}

	public void setLoginId(String loginId) {
		this.loginId = loginId;
	}

	/**
	 * This method provides a way for security officers to "reset" the password.
	 */
	void setPassword(String newPassword) throws BusinessException {

		// TODO - review. Do all tests of password size, content, history, etc
		// here...

		if (StringUtil.isEmpty(newPassword)) {
			throw new ValueRequiredException(this, "User_password");
		}

		if (_loaded_password != null && newPassword.equals(_loaded_password)) {
			throw new GenericBusinessException("User_newpassword_is_same");
		}

		// The following is not at all secure. Remember that this object will be
		// detached and returned to the web client, where this clear-text
		// password might become vulnerable.
		// Alternatives include encrypting the password before storing it,
		// saving it encrypted to a separate entity that is never detached, or
		// using an enterprise-strength security framework like JAAS, Acegi,
		// etc. MD5 encryption is planned for this example.

		this.password = newPassword;
	}

	public String getFirstName() {
		return firstName;
	}

	public void setFirstName(String firstName) {
		this.firstName = firstName;
	}

	public String getLastName() {
		return lastName;
	}

	public void setLastName(String lastName) {
		this.lastName = lastName;
	}

	public String getEmailAddress() {
		return emailAddress;
	}

	public void setEmailAddress(String emailAddress) {
		this.emailAddress = emailAddress;
	}

	public boolean isActive() {
		return active;
	}

	public void setActive(boolean active) {
		this.active = active;
	}

	public String getSalutation() {
		return salutation;
	}

	public void setSalutation(String salutation) {
		this.salutation = salutation;
	}

	public Date getExpirationDate() {
		return expirationDate;
	}

	public void setExpirationDate(Date expirationDate) {
		this.expirationDate = expirationDate;
	}

}
